<?php
// Include configuration and utility functions
require_once __DIR__ . '/config.php';
require_once __DIR__ . '/includes/utils.php';

// ---------- 0. Create a Lock Mechanism ----------
define('LOCK_DIR', __DIR__ . '/tmp');
define('LOCK_FILE', LOCK_DIR . '/fetch_hits.lock');
define('LOCK_TIMEOUT', 12 * 60 * 60); // 12 hours

// Auto-remove lock on script shutdown (CTRL+C, kill, fatal error)
register_shutdown_function('removeLock');

function createLock() {
    if (!is_dir(LOCK_DIR)) {
        mkdir(LOCK_DIR, 0755, true);
        echo "📂 [LOCK] Created lock directory: " . LOCK_DIR . "<br>";
        logMessage("📂 [LOCK] Created lock directory: " . LOCK_DIR);
    }

    if (file_exists(LOCK_FILE)) {
        $fileTime = filemtime(LOCK_FILE);
        $currentTime = time();

        if (($currentTime - $fileTime) < LOCK_TIMEOUT) {
            echo "🔒 [LOCK] Another instance is running. Exiting...<br>";
            logMessage("🔒 [LOCK] Another instance is running. Exiting...");
            exit;
        } else {
            echo "⚠️ [LOCK] Stale lock detected. Removing old lock...<br>";
            logMessage("⚠️ [LOCK] Stale lock detected. Removing old lock...");
            unlink(LOCK_FILE);
        }
    }

    file_put_contents(LOCK_FILE, getmypid());
    echo "🔑 [LOCK] Lock acquired. Starting process...<br>";
    logMessage("🔑 [LOCK] Lock acquired. Starting process...");
}

function removeLock() {
    if (file_exists(LOCK_FILE)) {
        unlink(LOCK_FILE);
        echo "🔓 [LOCK] Lock released.<br>";
        logMessage("🔓 [LOCK] Lock released.");
    }
}

// ---------- 1. Refresh Access Token ----------
function refreshToken() {
    echo "🔄 [TOKEN] Refreshing access token...<br>";
    logMessage("🔄 [TOKEN] Refreshing access token...");

    $ch = curl_init();
    curl_setopt($ch, CURLOPT_URL, 'https://www.ssgenius.com/mautic-data-fetcher/token_handler.php');
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
    curl_setopt($ch, CURLOPT_TIMEOUT, 30);

    $response = curl_exec($ch);
    $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
    curl_close($ch);

    echo "🔎 [TOKEN] Response: $response<br>";
    logMessage("🔎 [TOKEN] Response: $response");

    if ($httpCode === 200 && strpos($response, 'Authorization successful') !== false) {
        echo "✅ [TOKEN] Access token refreshed.<br>";
        logMessage("✅ [TOKEN] Access token refreshed.");
    } else {
        echo "❌ [TOKEN] Failed to refresh token.<br>";
        logMessage("❌ [TOKEN] Failed to refresh token.");
        removeLock();
        die("❌ [TOKEN] Token refresh failed.");
    }
}

// ---------- 2. Load Latest Access Token ----------
function loadAccessToken() {
    global $accessToken;

    echo "🔍 [TOKEN] Loading latest access token...<br>";
    logMessage("🔍 [TOKEN] Loading latest access token...");

    $conn = dbConnect();
    $result = $conn->query("SELECT access_token FROM emma_mason_api_tokens WHERE client_name='emma_mason' ORDER BY id DESC LIMIT 1");

    if ($result && $result->num_rows > 0) {
        $accessToken = $result->fetch_assoc()['access_token'];
        echo "🔓 [TOKEN] Access token loaded.<br>";
        logMessage("🔓 [TOKEN] Access token loaded.");
    } else {
        echo "❌ [TOKEN] Failed to load access token.<br>";
        logMessage("❌ [TOKEN] Failed to load access token.");
        removeLock();
        die("❌ [TOKEN] Failed to load access token.");
    }

    $conn->close();
}

// ---------- 3. Fetch and Insert Page Hits ----------
function fetchAndInsertHits($conn, $accessToken) {
    $limit = 100;
    $result = $conn->query("SELECT MAX(id) AS last_id FROM mautic_page_hits");
    $lastId = ($result && $result->num_rows > 0) ? (int)$result->fetch_assoc()['last_id'] : 0;

    echo "🔎 [FETCH] Starting page hits from ID: $lastId<br>";
    logMessage("🔎 [FETCH] Starting page hits from ID: $lastId");

    while (true) {
        $url = MAUTIC_BASE_URL . "/api/stats/page_hits?start=$lastId&limit=$limit";
        echo "📦 [FETCH] Fetching hits from URL: $url<br>";
        logMessage("📦 [FETCH] Fetching hits from URL: $url");

        $data = fetchData($url, $accessToken);
        if (empty($data['stats'])) {
            echo "🛑 [FETCH] No more hits to fetch.<br>";
            logMessage("🛑 [FETCH] No more hits to fetch.");
            break;
        }

        echo "✅ [FETCH] Fetched " . count($data['stats']) . " hits.<br>";
        logMessage("✅ [FETCH] Fetched " . count($data['stats']) . " hits.");

        saveHitsBatch($conn, $data['stats']);
        echo "💾 [DB] Saved hits batch.<br>";
        logMessage("💾 [DB] Saved hits batch.");

        $lastId += $limit;
    }
}

function saveHitsBatch($conn, $hits) {
    $stmt = $conn->prepare("
        INSERT IGNORE INTO mautic_page_hits (
            id, ip_id, date_hit, country, region, city, isp, organization, code, url, user_agent, browser_languages, tracking_id, query
        ) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
    ");

    foreach ($hits as $hit) {
        echo "➡️ [HIT] Processing hit ID: " . $hit['id'] . "<br>";
        logMessage("➡️ [HIT] Processing hit ID: " . $hit['id']);

        $stmt->bind_param(
            "iissssssssssss",
            $hit['id'], $hit['ip_id'], convertToMySQLDateTime($hit['date_hit']), // Updated here
            $hit['country'], $hit['region'], $hit['city'], $hit['isp'], $hit['organization'],
            $hit['code'], $hit['url'], $hit['user_agent'], $hit['browser_languages'],
            $hit['tracking_id'], $hit['query']
        );
        $stmt->execute();
    }

    $stmt->close();
}

// ---------- 4. Fetch Data from API ----------
function fetchData($url, $accessToken) {
    $ch = curl_init($url);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($ch, CURLOPT_HTTPHEADER, ["Authorization: Bearer $accessToken"]);
    $response = curl_exec($ch);
    curl_close($ch);

    return json_decode($response, true);
}

// ---------- 5. Execute the Process ----------
createLock();
refreshToken();
loadAccessToken();

$conn = dbConnect();
fetchAndInsertHits($conn, $accessToken);
$conn->close();

removeLock();
echo "🚀 [DONE] Page hits fetched and inserted successfully.<br>";
logMessage("🚀 [DONE] Page hits fetched and inserted successfully.");
?>
