<?php
// Include configuration and utility functions
require_once __DIR__ . '/config.php';
require_once __DIR__ . '/includes/utils.php';

// ---------- 0. Create a Lock Mechanism ----------
define('LOCK_DIR', __DIR__ . '/tmp');
define('LOCK_FILE', LOCK_DIR . '/fetch_redirects_hits.lock');
define('LOCK_TIMEOUT', 12 * 60 * 60); // 12 hours

// Auto-remove lock on script shutdown (CTRL+C, kill, fatal error)
register_shutdown_function('removeLock');

function createLock() {
    if (!is_dir(LOCK_DIR)) {
        mkdir(LOCK_DIR, 0755, true);
        logMessage("📂 [LOCK] Created lock directory: " . LOCK_DIR);
    }

    if (file_exists(LOCK_FILE)) {
        $fileTime = filemtime(LOCK_FILE);
        $currentTime = time();

        if (($currentTime - $fileTime) < LOCK_TIMEOUT) {
            logMessage("🔒 [LOCK] Another instance is running. Exiting...");
            exit;
        } else {
            logMessage("⚠️ [LOCK] Stale lock detected. Removing old lock...");
            unlink(LOCK_FILE);
        }
    }

    file_put_contents(LOCK_FILE, getmypid());
    logMessage("🔑 [LOCK] Lock acquired. Starting process...");
}

function removeLock() {
    if (file_exists(LOCK_FILE)) {
        unlink(LOCK_FILE);
        logMessage("🔓 [LOCK] Lock released.");
    }
}

// ---------- 1. Refresh Access Token ----------
function refreshToken() {
    logMessage("🔄 [TOKEN] Attempting to refresh the access token...");
    $ch = curl_init();
    curl_setopt($ch, CURLOPT_URL, 'https://www.ssgenius.com/mautic-data-fetcher/token_handler.php');
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
    curl_setopt($ch, CURLOPT_TIMEOUT, 30);

    $response = curl_exec($ch);
    $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
    curl_close($ch);

    if ($httpCode === 200 && strpos($response, 'Authorization successful') !== false) {
        logMessage("✅ [TOKEN] Access token refreshed successfully.");
    } else {
        logMessage("❌ [TOKEN] Failed to refresh token. Response: $response");
        removeLock();
        die("❌ [TOKEN] Token refresh failed.");
    }
}

// ---------- 2. Load Latest Access Token ----------
function loadAccessToken() {
    global $accessToken;

    logMessage("🔄 [TOKEN] Loading the latest access token...");
    $conn = dbConnect();
    $result = $conn->query("SELECT access_token FROM emma_mason_api_tokens WHERE client_name='emma_mason' ORDER BY id DESC LIMIT 1");

    if ($result && $result->num_rows > 0) {
        $accessToken = $result->fetch_assoc()['access_token'];
        logMessage("🔓 [TOKEN] Access token loaded: $accessToken");
    } else {
        logMessage("❌ [TOKEN] Failed to load access token.");
        removeLock();
        die("❌ [TOKEN] Failed to load access token.");
    }

    $conn->close();
}

// ---------- 3. Fetch and Insert Redirects ----------
function fetchAndInsertRedirects($conn, $accessToken) {
    $limit = 100;
    $result = $conn->query("SELECT MAX(id) AS last_id FROM mautic_page_redirects");
    $lastId = ($result && $result->num_rows > 0) ? (int)$result->fetch_assoc()['last_id'] : 0;

    logMessage("🔎 [FETCH] Starting redirects from ID: $lastId");

    while (true) {
        $url = MAUTIC_BASE_URL . "/api/stats/page_redirects?start=$lastId&limit=$limit";
        logMessage("🌐 [API] Fetching redirects from: $url");

        $data = fetchData($url, $accessToken);
        if (empty($data['stats'])) {
            logMessage("🛑 [FETCH] No more redirects to fetch.");
            break;
        }

        logMessage("📥 [FETCH] Retrieved " . count($data['stats']) . " redirects.");
        saveRedirectsBatch($conn, $data['stats']);
        $lastId += $limit;
    }
}

function saveRedirectsBatch($conn, $redirects) {
    logMessage("💾 [SAVE] Saving " . count($redirects) . " redirect records...");
    $stmt = $conn->prepare("
        INSERT IGNORE INTO mautic_page_redirects (
            id, is_published, date_added, created_by, created_by_user, redirect_id, url, hits, unique_hits
        ) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)
    ");

    foreach ($redirects as $redirect) {
        $stmt->bind_param(
            "iisssssii",
            $redirect['id'], $redirect['is_published'], $redirect['date_added'],
            $redirect['created_by'], $redirect['created_by_user'], $redirect['redirect_id'],
            $redirect['url'], $redirect['hits'], $redirect['unique_hits']
        );
        $stmt->execute();
    }

    logMessage("✅ [SAVE] Redirects saved successfully.");
    $stmt->close();
}

// ---------- 4. Fetch and Insert Page Hits ----------
function fetchAndInsertHits($conn, $accessToken) {
    $limit = 100;
    $result = $conn->query("SELECT MAX(id) AS last_id FROM mautic_page_hits");
    $lastId = ($result && $result->num_rows > 0) ? (int)$result->fetch_assoc()['last_id'] : 0;

    logMessage("🔎 [FETCH] Starting page hits from ID: $lastId");

    while (true) {
        $url = MAUTIC_BASE_URL . "/api/stats/page_hits?start=$lastId&limit=$limit";
        logMessage("🌐 [API] Fetching page hits from: $url");

        $data = fetchData($url, $accessToken);
        if (empty($data['stats'])) {
            logMessage("🛑 [FETCH] No more page hits to fetch.");
            break;
        }

        logMessage("📥 [FETCH] Retrieved " . count($data['stats']) . " page hits.");
        saveHitsBatch($conn, $data['stats']);
        $lastId += $limit;
    }
}

function saveHitsBatch($conn, $hits) {
    logMessage("💾 [SAVE] Saving " . count($hits) . " page hit records...");
    $stmt = $conn->prepare("
        INSERT IGNORE INTO mautic_page_hits (
            id, ip_id, date_hit, country, region, city, isp, organization, code, url, user_agent, browser_languages, tracking_id, query
        ) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
    ");

    foreach ($hits as $hit) {
        $stmt->bind_param(
            "iissssssssssss",
            $hit['id'], $hit['ip_id'], convertToMySQLDateTime($hit['date_hit']),
            $hit['country'], $hit['region'], $hit['city'], $hit['isp'], $hit['organization'],
            $hit['code'], $hit['url'], $hit['user_agent'], $hit['browser_languages'],
            $hit['tracking_id'], $hit['query']
        );
        $stmt->execute();
    }

    logMessage("✅ [SAVE] Page hits saved successfully.");
    $stmt->close();
}

// ---------- 5. Fetch Data from API ----------
function fetchData($url, $accessToken) {
    $ch = curl_init($url);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($ch, CURLOPT_HTTPHEADER, ["Authorization: Bearer $accessToken"]);
    $response = curl_exec($ch);
    curl_close($ch);

    return json_decode($response, true);
}

// ---------- 6. Execute the Process ----------
logMessage("🚀 [START] Process started.");
createLock();
refreshToken();
loadAccessToken();

$conn = dbConnect();
fetchAndInsertRedirects($conn, $accessToken);
fetchAndInsertHits($conn, $accessToken);
$conn->close();

logMessage("🎉 [DONE] Finished fetching redirects and hits. Starting JSON details processing...");

// Call the next script
require_once __DIR__ . '/includes/populate_parsed_json_details.php';

removeLock();
logMessage("🎉 [FINAL DONE] All processes completed successfully.");
