<?php
// Include configuration and utility functions
require_once __DIR__ . '/config.php';
require_once __DIR__ . '/includes/utils.php';

// ---------- 0. Create a Lock Mechanism ----------
define('LOCK_DIR', __DIR__ . '/tmp');
define('LOCK_FILE', LOCK_DIR . '/fetch_stats_email_full.lock');
define('LOCK_TIMEOUT', 12 * 60 * 60); // 12 hours

// Auto-remove lock on script shutdown (CTRL+C, kill, fatal error)
register_shutdown_function('removeLock');

function createLock() {
    // Ensure the /tmp directory exists
    if (!is_dir(LOCK_DIR)) {
        mkdir(LOCK_DIR, 0755, true);
        echo "📂 [LOCK] Created lock directory: " . LOCK_DIR . "<br>";
        logMessage("📂 [LOCK] Created lock directory: " . LOCK_DIR);
    }

    if (file_exists(LOCK_FILE)) {
        $fileTime = filemtime(LOCK_FILE);
        $currentTime = time();

        if (($currentTime - $fileTime) < LOCK_TIMEOUT) {
            echo "🔒 [LOCK] Another instance is running. Exiting...<br>";
            logMessage("🔒 [LOCK] Another instance is running. Exiting...");
            exit;
        } else {
            echo "⚠️ [LOCK] Stale lock detected. Removing old lock...<br>";
            logMessage("⚠️ [LOCK] Stale lock detected. Removing old lock...");
            unlink(LOCK_FILE);
        }
    }

    file_put_contents(LOCK_FILE, getmypid());
    echo "🔑 [LOCK] Lock acquired. Starting process...<br>";
    logMessage("🔑 [LOCK] Lock acquired. Starting process...");
}

function removeLock() {
    if (file_exists(LOCK_FILE)) {
        unlink(LOCK_FILE);
        echo "🔓 [LOCK] Lock released.<br>";
        logMessage("🔓 [LOCK] Lock released.");
    }
}


// ---------- 1. Refresh Access Token ----------
function refreshToken() {
    echo "🔄 [TOKEN] Refreshing access token...<br>";
    logMessage("🔄 [TOKEN] Refreshing access token...");

    $ch = curl_init();
    curl_setopt($ch, CURLOPT_URL, 'https://www.ssgenius.com/mautic-data-fetcher/token_handler.php');
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
    curl_setopt($ch, CURLOPT_TIMEOUT, 30);

    $response = curl_exec($ch);
    $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
    $error = curl_error($ch);
    curl_close($ch);

    echo "🔎 [TOKEN] Response: $response<br>";
    logMessage("🔎 [TOKEN] Response: $response");

    if ($httpCode === 200 && strpos($response, 'Authorization successful') !== false) {
        logMessage("🔑 [TOKEN] Access token refreshed.");
        echo "✅ [TOKEN] Token refreshed.<br>";
    } else {
        logMessage("❌ [TOKEN] Failed to refresh token. HTTP Code: $httpCode. Error: $error");
        removeLock();  // 🔓 Release lock on failure
        die("❌ [TOKEN] Token refresh failed.");
    }
}

// ---------- 2. Load Latest Access Token ----------
function loadAccessToken() {
    global $accessToken;

    $conn = dbConnect();
    $result = $conn->query("SELECT access_token FROM emma_mason_api_tokens WHERE client_name='emma_mason' ORDER BY id DESC LIMIT 1");

    if ($result && $result->num_rows > 0) {
        $accessToken = $result->fetch_assoc()['access_token'];
        logMessage("🔓 [TOKEN] Access token loaded.");
        echo "🔓 [TOKEN] Access token loaded.<br>";
    } else {
        logMessage("❌ [TOKEN] Failed to load access token.");
        removeLock();  // 🔓 Release lock on failure
        die("❌ [TOKEN] Failed to load access token.");
    }

    $conn->close();
}

// ---------- 3. Fetch and Insert Email Stats ----------
function fetchAndInsertEmailStats() {
    global $accessToken;
    $limit = 500;

    $conn = dbConnect();

    // Get the last inserted ID
    $result = $conn->query("SELECT MAX(id) AS last_id FROM mautic_stats_email_full");
    $lastId = ($result && $result->num_rows > 0) ? (int)$result->fetch_assoc()['last_id'] : 0;

    echo "🔎 [FETCH] Starting from record ID: $lastId<br>";
    logMessage("🔎 [FETCH] Starting from record ID: $lastId");

    while (true) {
        $url = MAUTIC_BASE_URL . "/api/stats/email_stats?start=$lastId&limit=$limit";
        echo "📦 [FETCH] Fetching batch from URL: $url<br>";
        logMessage("📦 [FETCH] Fetching batch from URL: $url");

        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_HTTPHEADER, ["Authorization: Bearer $accessToken"]);
        $response = curl_exec($ch);
        $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        curl_close($ch);

        if ($httpCode === 200) {
            $data = json_decode($response, true);
            $records = $data['stats'] ?? [];

            if (empty($records)) {
                echo "🛑 [FETCH] No more data to fetch.<br>";
                logMessage("🛑 [FETCH] No more data to fetch.");
                break;
            }

            saveEmailStatsBatch($conn, $records);
            $lastId += $limit;
        } else {
            logMessage("❌ [FETCH] Error fetching data. HTTP Code: $httpCode");
            echo "❌ [FETCH] Error fetching data. HTTP Code: $httpCode<br>";
            break;
        }
    }

    $conn->close();
}

// ---------- 4. Insert Data into mautic_stats_email_full ----------
function saveEmailStatsBatch($conn, $records) {
    $stmt = $conn->prepare("
        INSERT IGNORE INTO mautic_stats_email_full (
            id, email_id, lead_id, list_id, email_address, date_sent, is_read,
            date_read, is_failed, viewed_in_browser, tracking_hash, retry_count,
            source, source_id, open_count, last_opened, open_details, ip_id
        ) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
    ");

    foreach ($records as $index => $record) {
        $id = (int)($record['id'] ?? 0);
        $email_id = (int)($record['email_id'] ?? 0);
        $lead_id = (int)($record['lead_id'] ?? 0);
        $list_id = (int)($record['list_id'] ?? 0);
        $email_address = $record['email_address'] ?? '';
        $date_sent = $record['date_sent'] ?? null;
        $is_read = (int)($record['is_read'] ?? 0);
        $date_read = $record['date_read'] ?? null;
        $is_failed = (int)($record['is_failed'] ?? 0);
        $viewed_in_browser = (int)($record['viewed_in_browser'] ?? 0);
        $tracking_hash = $record['tracking_hash'] ?? '';
        $retry_count = (int)($record['retry_count'] ?? 0);
        $source = $record['source'] ?? '';
        $source_id = (int)($record['source_id'] ?? 0);
        $open_count = (int)($record['open_count'] ?? 0);
        $last_opened = $record['last_opened'] ?? null;
        $open_details = $record['open_details'] ?? '';
        $ip_id = (int)($record['ip_id'] ?? 0);

        $stmt->bind_param("iiiissssisisssiiss", $id, $email_id, $lead_id, $list_id, $email_address, $date_sent, $is_read, $date_read, $is_failed, $viewed_in_browser, $tracking_hash, $retry_count, $source, $source_id, $open_count, $last_opened, $open_details, $ip_id);
        $stmt->execute();
    }

    $stmt->close();
}

// ---------- 5. Execute the Process ----------
createLock();  // 🔒 Lock before starting

refreshToken();
loadAccessToken();
fetchAndInsertEmailStats();

removeLock();  // 🔓 Unlock after completion

echo "🚀 [DONE] Data fetching completed.<br>";
?>
