<?php
require_once __DIR__ . '/../config.php';
require_once __DIR__ . '/utils.php';

// Connect to the database
$conn = new mysqli(DB_HOST, DB_USER, DB_PASSWORD, DB_NAME);

if ($conn->connect_error) {
    logMessage("❌ [DB] Connection failed: " . $conn->connect_error);
    die("Connection failed: " . $conn->connect_error);
}

logMessage("🚀 [PROCESS] Starting the daily email activity update...");

// STEP 1: Update the Calendar Table
logMessage("🔄 [STEP 1] Updating calendar table...");

$updateCalendarSQL = "
    INSERT IGNORE INTO calendar (calendar_date)
    SELECT sub.earliest_date + INTERVAL seq.day DAY
    FROM 
    (
        SELECT LEAST(
            (SELECT MIN(click_date) FROM parsed_click_data WHERE click_date IS NOT NULL),
            (SELECT MIN(date_sent)  FROM mautic_stats_email_full WHERE date_sent IS NOT NULL)
        ) AS earliest_date
    ) AS sub
    CROSS JOIN
    (
        SELECT 0 AS day UNION ALL SELECT 1 UNION ALL SELECT 2 UNION ALL
        SELECT 3 UNION ALL SELECT 4 UNION ALL SELECT 5 UNION ALL SELECT 6
    ) AS seq
    WHERE sub.earliest_date IS NOT NULL
      AND sub.earliest_date + INTERVAL seq.day DAY <= CURDATE();
";

if ($conn->query($updateCalendarSQL) === TRUE) {
    logMessage("✅ [STEP 1] Calendar table updated.");
} else {
    logMessage("❌ [STEP 1] Error: " . $conn->error);
    logMessage("SQL: " . $updateCalendarSQL);
}

// STEP 2: Aggregate Click Data
logMessage("🔄 [STEP 2] Aggregating click data...");

$aggregateClickDataSQL = "
    REPLACE INTO aggregated_click_data (
        click_date, 
        email_id, 
        campaign_name, 
        total_clicks, 
        total_different_leads_that_clicked
    )
    SELECT 
        DATE(click_date) AS click_date,
        email_id,
        campaign_id AS campaign_name,
        COUNT(*) AS total_clicks, -- Total clicks per email and day
        COUNT(DISTINCT lead_id) AS total_different_leads_that_clicked -- Unique leads clicking per email and day
    FROM parsed_click_data
    WHERE email_id IS NOT NULL
      AND campaign_id IS NOT NULL
    GROUP BY DATE(click_date), email_id, campaign_id;
";

logMessage("Executing query: " . $aggregateClickDataSQL);
if ($conn->query($aggregateClickDataSQL) === TRUE) {
    logMessage("✅ [STEP 2] Click data aggregated.");
} else {
    logMessage("❌ [STEP 2] Error: " . $conn->error);
    logMessage("SQL: " . $aggregateClickDataSQL);
}

// STEP 3: Aggregate Sent Data
logMessage("🔄 [STEP 3] Aggregating sent data...");

$aggregateSentDataSQL = "
    REPLACE INTO aggregated_sent_data (
        date_sent, 
        email_id, 
        campaign_id, 
        event_id, 
        total_sent, 
        total_opens, 
        total_bounces
    )
    SELECT 
        CAST(msf.date_sent AS DATE) AS date_sent,
        msf.email_id,
        COALESCE(emce.campaign_id, 0) AS campaign_id,
        COALESCE(emce.event_id, 0) AS event_id,
        COUNT(msf.id) AS total_sent,
        SUM(msf.is_read) AS total_opens,
        SUM(msf.is_failed) AS total_bounces
    FROM mautic_stats_email_full msf
    LEFT JOIN emma_mason_campaign_events emce
        ON msf.email_id = emce.channel_id
        AND emce.type = 'email.send'
    WHERE msf.email_id IS NOT NULL
    GROUP BY CAST(msf.date_sent AS DATE), msf.email_id, emce.campaign_id, emce.event_id;
";

logMessage("Executing query: " . $aggregateSentDataSQL);
if ($conn->query($aggregateSentDataSQL) === TRUE) {
    logMessage("✅ [STEP 3] Sent data aggregated.");
} else {
    logMessage("❌ [STEP 3] Error: " . $conn->error);
    logMessage("SQL: " . $aggregateSentDataSQL);
}

// STEP 4.1: Insert all records from aggregated_sent_data
logMessage("🔄 [STEP 4.1] Inserting records from aggregated_sent_data...");
$insertSentDataSQL = "
    REPLACE INTO email_activity_per_day (
        calendar_date, 
        email_id, 
        campaign_id, 
        total_sent, 
        total_opens, 
        total_bounces, 
        total_clicks, 
        total_different_leads_that_clicked, 
        date_sent
    )
    SELECT 
        date_sent AS calendar_date,
        email_id,
        campaign_id,
        total_sent,
        total_opens,
        total_bounces,
        0 AS total_clicks, -- Default to 0
        0 AS total_different_leads_that_clicked, -- Default to 0
        date_sent
    FROM aggregated_sent_data;
";
if ($conn->query($insertSentDataSQL) === TRUE) {
    logMessage("✅ [STEP 4.1] Records from aggregated_sent_data inserted.");
} else {
    logMessage("❌ [STEP 4.1] Error: " . $conn->error);
}

// STEP 4.2: Update records with matching data from aggregated_click_data
logMessage("🔄 [STEP 4.2] Updating records with matching data from aggregated_click_data...");
$updateClickDataSQL = "
    UPDATE email_activity_per_day eap
    LEFT JOIN aggregated_click_data acd
        ON eap.calendar_date = acd.click_date
        AND eap.email_id = acd.email_id
    SET 
        eap.total_clicks = COALESCE(acd.total_clicks, 0),
        eap.total_different_leads_that_clicked = COALESCE(acd.total_different_leads_that_clicked, 0)
    WHERE acd.email_id IS NOT NULL;
";
if ($conn->query($updateClickDataSQL) === TRUE) {
    logMessage("✅ [STEP 4.2] Records updated with aggregated_click_data.");
} else {
    logMessage("❌ [STEP 4.2] Error: " . $conn->error);
}

// STEP 4.3: Insert missing records from aggregated_click_data
logMessage("🔄 [STEP 4.3] Inserting missing records from aggregated_click_data...");
$insertMissingClickDataSQL = "
    INSERT IGNORE INTO email_activity_per_day (
        calendar_date, 
        email_id, 
        campaign_id, 
        total_sent, 
        total_opens, 
        total_bounces, 
        total_clicks, 
        total_different_leads_that_clicked, 
        date_sent
    )
    SELECT 
        click_date AS calendar_date,
        email_id,
        0 AS campaign_id, -- Default to 0 if not matched
        0 AS total_sent, -- Default to 0 if no sent data
        0 AS total_opens, -- Default to 0 if no sent data
        0 AS total_bounces, -- Default to 0 if no sent data
        total_clicks,
        total_different_leads_that_clicked,
        click_date AS date_sent
    FROM aggregated_click_data acd
    WHERE NOT EXISTS (
        SELECT 1 
        FROM email_activity_per_day eap
        WHERE eap.calendar_date = acd.click_date
          AND eap.email_id = acd.email_id
    );
";

if ($conn->query($insertMissingClickDataSQL) === TRUE) {
    logMessage("✅ [STEP 4.3] Missing records from aggregated_click_data inserted.");
} else {
    logMessage("❌ [STEP 4.3] Error: " . $conn->error);
}

// STEP 5: Update campaign_id_numeric in email_activity_per_day
logMessage("🔄 [STEP 5] Updating campaign_id_numeric in email_activity_per_day...");

$updateCampaignIdNumericSQL = "
    UPDATE email_activity_per_day eap
    LEFT JOIN emma_mason_campaign_events emce
        ON eap.email_id = emce.channel_id
        AND emce.type = 'email.send'
    SET eap.campaign_id_numeric = COALESCE(emce.campaign_id, 0);
";

logMessage("Executing query: " . $updateCampaignIdNumericSQL);
if ($conn->query($updateCampaignIdNumericSQL) === TRUE) {
    logMessage("✅ [STEP 5] campaign_id_numeric updated in email_activity_per_day with default 0 for unmatched rows.");
} else {
    logMessage("❌ [STEP 5] Error: " . $conn->error);
    logMessage("SQL: " . $updateCampaignIdNumericSQL);
}

/** STEP 6 is not working with this logic
// STEP 6: Extract `original_id` values and their details for processing
logMessage("🔄 [STEP 6] Extracting `original_id` values and details for processing...");

// Define batch size
$batchSize = 100;

// Get the last processed ID from `detailed_email_click_data`
$lastProcessedIdResult = $conn->query("SELECT MAX(original_id) AS last_id FROM detailed_email_click_data");
$lastProcessedId = $lastProcessedIdResult->fetch_assoc()['last_id'] ?? 0;

logMessage("🔎 [DETAILS] Last processed `original_id`: $lastProcessedId");

// Fetch the next 100 IDs to process
$sqlIds = "
    SELECT DISTINCT original_id
    FROM parsed_json_details
    WHERE original_id > $lastProcessedId
    ORDER BY original_id ASC
    LIMIT $batchSize;
";

$idResult = $conn->query($sqlIds);

if ($idResult->num_rows > 0) {
    $ids = [];
    while ($row = $idResult->fetch_assoc()) {
        $ids[] = $row['original_id'];
    }

    $idList = implode(", ", $ids);
    logMessage("✅ [NEXT IDS] Found 100 IDs to process: $idList");

    // Fetch details for these IDs
    $sqlDetails = "
        SELECT 
            mph.date_hit AS click_date,
            msf.email_id,
            msf.email_subject AS campaign_name,
            pjd.key_value AS lead_id
        FROM parsed_json_details pjd
        LEFT JOIN mautic_page_hits mph 
            ON pjd.original_id = mph.id
        LEFT JOIN mautic_stats_email_full msf 
            ON mph.tracking_id = msf.tracking_hash
        WHERE pjd.original_id IN ($idList)
          AND pjd.key_name = 'ct.lead'
          AND pjd.key_value IS NOT NULL
          AND msf.email_id IS NOT NULL;
    ";

    $detailsResult = $conn->query($sqlDetails);

    if ($detailsResult->num_rows > 0) {
        while ($row = $detailsResult->fetch_assoc()) {
            logMessage("🔍 [DETAILS] Record: Click Date = {$row['click_date']}, Email ID = {$row['email_id']}, Campaign Name = {$row['campaign_name']}, Lead ID = {$row['lead_id']}");
        }
    } else {
        logMessage("🛑 [DETAILS] No details found for the selected IDs.");
    }
} else {
    logMessage("🛑 [NEXT IDS] No IDs found to process after `original_id`: $lastProcessedId");
}

// Finish
logMessage("🎉 [STEP 6] Completed ID extraction and detail logging.");

End of STEP 6 comment **/



// Finish
logMessage("🎉 [PROCESS] Daily email activity update completed.");
$conn->close();
