<?php
// Include configuration and utility functions
require_once __DIR__ . '/../config.php';

// *****************************************
// * Full Campaign Fetch & Save *
// *****************************************

function getLastFetchedCampaignId() {
    $conn = dbConnect();
    $result = $conn->query("SELECT MAX(campaign_id) AS last_id FROM emma_mason_campaign_details");
    $lastId = $result ? $result->fetch_assoc()['last_id'] : 0;
    $conn->close();

    return $lastId ?? 0;
}

function fetchAndSaveCampaigns() {
    global $accessToken;

    $lastFetchedId = getLastFetchedCampaignId();
    logMessage("🔍 Last fetched campaign ID: $lastFetchedId");
    echo "🔍 Last fetched campaign ID: $lastFetchedId<br>";

    $currentId = $lastFetchedId + 1;
    $emptyFetchCount = 0;

    while (true) {
        $url = MAUTIC_BASE_URL . "/api/campaigns/$currentId";
        logMessage("📦 Fetching campaign ID: $currentId from URL: $url");
        echo "📦 Fetching campaign ID: $currentId from URL: $url<br>";

        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_HTTPHEADER, ["Authorization: Bearer $accessToken"]);
        $response = curl_exec($ch);
        $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        curl_close($ch);

        if ($httpCode === 200) {
            $data = json_decode($response, true);
            if (!empty($data['campaign'])) {
                saveCampaign($data['campaign']);
                $emptyFetchCount = 0;
            } else {
                logMessage("⚠️ No campaign found for ID: $currentId");
                $emptyFetchCount++;
            }
        } elseif ($httpCode === 404) {
            logMessage("⚠️ Campaign ID $currentId not found. Continuing...");
            $emptyFetchCount++;
        } else {
            logMessage("❌ Failed to fetch campaign ID $currentId. HTTP Code: $httpCode");
            echo "❌ Failed to fetch campaign ID $currentId. HTTP Code: $httpCode<br>";
            break;
        }

        if ($emptyFetchCount >= 5) {
            logMessage("🛑 No more campaigns found. Stopping fetch.");
            echo "🛑 No more campaigns found. Stopping fetch.<br>";
            break;
        }

        $currentId++;
    }

    logMessage("✅ Finished fetching campaigns.");
    echo "✅ Finished fetching campaigns.<br>";
}

// Save campaign details with complete data
function saveCampaign($campaign) {
    $conn = dbConnect();

    $stmt = $conn->prepare("
        REPLACE INTO emma_mason_campaign_details (
            campaign_id, name, description, status, category, allow_restart, 
            publish_up, publish_down, created_by, modified_by, date_added, date_modified
        ) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
    ");

    $status = $campaign['isPublished'] ? 'active' : 'inactive';

    // Convert dates to MySQL format
    $publishUp = !empty($campaign['publishUp']) ? convertToMySQLDateTime($campaign['publishUp']) : null;
    $publishDown = !empty($campaign['publishDown']) ? convertToMySQLDateTime($campaign['publishDown']) : null;
    $dateAdded = !empty($campaign['dateAdded']) ? convertToMySQLDateTime($campaign['dateAdded']) : null;
    $dateModified = !empty($campaign['dateModified']) ? convertToMySQLDateTime($campaign['dateModified']) : null;

    // Sanitize data to avoid arrays being passed into SQL
    $campaignId = (int) $campaign['id'];
    $name = is_string($campaign['name']) ? $campaign['name'] : json_encode($campaign['name']);
    $description = is_string($campaign['description']) ? $campaign['description'] : json_encode($campaign['description']);
    $category = is_string($campaign['category']) ? $campaign['category'] : json_encode($campaign['category']);
    $allowRestart = isset($campaign['allowRestart']) ? (int) $campaign['allowRestart'] : 0;
    $createdByUser = is_string($campaign['createdByUser']) ? $campaign['createdByUser'] : json_encode($campaign['createdByUser']);
    $modifiedByUser = is_string($campaign['modifiedByUser']) ? $campaign['modifiedByUser'] : json_encode($campaign['modifiedByUser']);

    // Bind sanitized data
    $stmt->bind_param(
        "issssissssss",
        $campaignId,
        $name,
        $description,
        $status,
        $category,
        $allowRestart,
        $publishUp,
        $publishDown,
        $createdByUser,
        $modifiedByUser,
        $dateAdded,
        $dateModified
    );

    // Execute and log result
    if ($stmt->execute()) {
        logMessage("✅ Saved Campaign ID: {$campaign['id']} - {$campaign['name']}");
        echo "✅ Saved Campaign ID: {$campaign['id']} - {$campaign['name']}<br>";

        if (!empty($campaign['events'])) {
            saveCampaignEvents($conn, $campaign['id'], $campaign['events']);
        }
    } else {
        logMessage("❌ Failed to save Campaign ID: {$campaign['id']} - Error: " . $stmt->error);
    }

    $conn->close();
}


// Save campaign events
function saveCampaignEvents($conn, $campaignId, $events) {
    foreach ($events as $event) {
        $stmt = $conn->prepare("
            REPLACE INTO emma_mason_campaign_events (
                event_id, campaign_id, name, type, event_type, channel, channel_id, `event_order`
            ) VALUES (?, ?, ?, ?, ?, ?, ?, ?)
        ");

        $stmt->bind_param(
            "iisssiii",
            $event['id'],
            $campaignId,
            $event['name'],
            $event['type'],
            $event['eventType'],
            $event['channel'],
            $event['channelId'],
            $event['order']
        );

        if ($stmt->execute()) {
            logMessage("📌 Event saved: Event ID {$event['id']} for Campaign ID $campaignId");

            if (!empty($event['properties'])) {
                saveEventProperties($conn, $event['id'], $event['properties']);
            }

            if (!empty($event['properties']['tags'])) {
                saveEventTags($conn, $event['id'], $campaignId, $event['properties']['tags']);
            }
        } else {
            logMessage("❌ Failed to save Event ID: {$event['id']}");
        }
    }
}

// Save event properties
function saveEventProperties($conn, $eventId, $properties) {
    foreach ($properties as $key => $value) {
        if (is_array($value)) {
            $value = json_encode($value);
        }

        $stmt = $conn->prepare("
            REPLACE INTO emma_mason_campaign_event_properties (event_id, property_key, property_value)
            VALUES (?, ?, ?)
        ");

        $stmt->bind_param("iss", $eventId, $key, $value);

        if ($stmt->execute()) {
            logMessage("⚙️ Property saved for Event ID $eventId: $key = $value");
        } else {
            logMessage("❌ Failed to save property for Event ID: $eventId");
        }
    }
}

// Save event tags
function saveEventTags($conn, $eventId, $campaignId, $tags) {
    foreach ($tags as $tag) {
        $stmt = $conn->prepare("
            REPLACE INTO emma_mason_campaign_tags (event_id, campaign_id, tag_name)
            VALUES (?, ?, ?)
        ");

        $stmt->bind_param("iis", $eventId, $campaignId, $tag);

        if ($stmt->execute()) {
            logMessage("🏷️ Tag saved: $tag (Event ID: $eventId, Campaign ID: $campaignId)");
        } else {
            logMessage("❌ Failed to save tag: $tag for Event ID: $eventId");
        }
    }
}
?>
