<?php
/**
 * Updates campaign_id in click_date_and_total_sales_report
 * using campaign_id from emma_mason_campaign_events where type = 'email.send'.
 */

require_once __DIR__ . '/../config.php';
require_once __DIR__ . '/utils.php';

// -----------------------------------------------
// Function to print logs in HTML
// -----------------------------------------------
function logCustomMessage($message) {
    echo "<p>" . htmlspecialchars($message) . "</p>\n";
}

// -----------------------------------------------
// Database Connection
// -----------------------------------------------
$conn = new mysqli(DB_HOST, DB_USER, DB_PASSWORD, DB_NAME);
if ($conn->connect_error) {
    die("<p style='color:red;'>❌ Database connection error: " . $conn->connect_error . "</p>");
}
$conn->set_charset("utf8mb4");

// -----------------------------------------------
// Step 1: Reset campaign_id to 0 before updating
// -----------------------------------------------
logCustomMessage("<strong>Step 1:</strong> Resetting all campaign_id values to 0");

$sqlReset = "UPDATE click_date_and_total_sales_report SET campaign_id = 0";
if ($conn->query($sqlReset) === TRUE) {
    logCustomMessage("✅ All campaign_id values reset to 0.");
} else {
    logCustomMessage("❌ SQL Error resetting campaign_id: " . $conn->error);
}

// -----------------------------------------------
// Step 2: Retrieve all matching email_id and campaign_id pairs
// -----------------------------------------------
logCustomMessage("<strong>Step 2:</strong> Retrieving all matching email_id and campaign_id pairs");

$sqlRetrieve = "SELECT cdsr.email_id, cdsr.id AS report_id, emce.campaign_id 
                FROM click_date_and_total_sales_report cdsr
                INNER JOIN emma_mason_campaign_events emce
                ON cdsr.email_id = emce.channel_id
                WHERE emce.type = 'email.send'";

$resultRetrieve = $conn->query($sqlRetrieve);

$recordsToUpdate = [];
if ($resultRetrieve && $resultRetrieve->num_rows > 0) {
    while ($row = $resultRetrieve->fetch_assoc()) {
        $recordsToUpdate[] = [
            'report_id' => $row['report_id'],
            'email_id' => $row['email_id'],
            'campaign_id' => $row['campaign_id']
        ];
    }
}

// Debug: Print all records before updating
logCustomMessage("🔎 Debug: Records to Update -> " . json_encode($recordsToUpdate));

// -----------------------------------------------
// Step 3: Update campaign_id in click_date_and_total_sales_report
// -----------------------------------------------
logCustomMessage("<strong>Step 3:</strong> Updating campaign_id for all matching records");

$updatedRows = 0;
foreach ($recordsToUpdate as $record) {
    logCustomMessage("🔄 Updating report_id: {$record['report_id']} with campaign_id: {$record['campaign_id']}");

    $sqlUpdate = "UPDATE click_date_and_total_sales_report SET campaign_id = ? WHERE id = ?";
    $stmt = $conn->prepare($sqlUpdate);
    
    if (!$stmt) {
        logCustomMessage("⚠️ [ERROR] SQL Prepare Failed: " . $conn->error);
        continue;
    }

    $stmt->bind_param("ii", $record['campaign_id'], $record['report_id']);
    
    if (!$stmt->execute()) {
        logCustomMessage("⚠️ [ERROR] Failed to update report_id: {$record['report_id']} - " . $stmt->error);
    } else {
        logCustomMessage("✅ Successfully updated report_id: {$record['report_id']} with campaign_id: {$record['campaign_id']}");
        $updatedRows += $stmt->affected_rows;
    }

    $stmt->close();
}

logCustomMessage("✅ campaign_id updated successfully. Rows affected: " . $updatedRows);


// -----------------------------------------------
// Step 4: Preview latest_order_status using lead_id (Start from record 10,000, limit 200)
// -----------------------------------------------
logCustomMessage("<strong>Step 4:</strong> Previewing latest_order_status for 200 records starting from record 10,000");

$sqlPreviewLeads = "SELECT cdsr.lead_id, cdsr.id AS report_id, ml.latest_order_status 
                      FROM click_date_and_total_sales_report cdsr
                      INNER JOIN mautic_leads ml
                      ON cdsr.lead_id = ml.id
                      WHERE ml.latest_order_status IS NOT NULL AND ml.latest_order_status <> ''
                      LIMIT 200 OFFSET 10000";

$resultPreviewLeads = $conn->query($sqlPreviewLeads);

if ($resultPreviewLeads && $resultPreviewLeads->num_rows > 0) {
    echo "<table border='1'><tr><th>Report ID</th><th>Lead ID</th><th>Latest Order Status</th></tr>";
    while ($row = $resultPreviewLeads->fetch_assoc()) {
        echo "<tr><td>{$row['report_id']}</td><td>{$row['lead_id']}</td><td>{$row['latest_order_status']}</td></tr>\n";
    }
    echo "</table>";
} else {
    logCustomMessage("⚠️ No latest_order_status records found in the given range.");
}

// -----------------------------------------------
// Step 5: Update latest_order_status in click_date_and_total_sales_report
// -----------------------------------------------
logCustomMessage("<strong>Step 5:</strong> Updating latest_order_status for all matching lead_id records");

$sqlRetrieveLeads = "SELECT cdsr.lead_id, cdsr.id AS report_id, ml.latest_order_status 
                      FROM click_date_and_total_sales_report cdsr
                      INNER JOIN mautic_leads ml
                      ON cdsr.lead_id = ml.id
                      WHERE ml.latest_order_status IS NOT NULL AND ml.latest_order_status <> ''";

$resultRetrieveLeads = $conn->query($sqlRetrieveLeads);

$leadRecordsToUpdate = [];
if ($resultRetrieveLeads && $resultRetrieveLeads->num_rows > 0) {
    while ($row = $resultRetrieveLeads->fetch_assoc()) {
        $leadRecordsToUpdate[] = [
            'report_id' => $row['report_id'],
            'lead_id' => $row['lead_id'],
            'latest_order_status' => $row['latest_order_status']
        ];
    }
}

$updatedLeadRows = 0;
foreach ($leadRecordsToUpdate as $record) {
    $sqlUpdateLead = "UPDATE click_date_and_total_sales_report SET latest_order_status = ? WHERE id = ?";
    $stmt = $conn->prepare($sqlUpdateLead);
    $stmt->bind_param("si", $record['latest_order_status'], $record['report_id']);
    if ($stmt->execute()) {
        $updatedLeadRows += $stmt->affected_rows;
    }
    $stmt->close();
}

logCustomMessage("✅ latest_order_status updated successfully. Rows affected: " . $updatedLeadRows);

// -----------------------------------------------
// Step 6: Retrieve and Display first_send_date and last_send_date
// -----------------------------------------------
logCustomMessage("<strong>Step 6:</strong> Retrieving first and last email send dates from email_activity_per_day");

$sendDateRecords = [];
$sqlRetrieveSendDates = "SELECT email_id, MIN(date_sent) AS first_send_date, MAX(date_sent) AS last_send_date 
                         FROM email_activity_per_day 
                         WHERE email_id IN (SELECT DISTINCT email_id FROM click_date_and_total_sales_report) 
                         GROUP BY email_id";

$resultRetrieveSendDates = $conn->query($sqlRetrieveSendDates);

if ($resultRetrieveSendDates && $resultRetrieveSendDates->num_rows > 0) {
    while ($row = $resultRetrieveSendDates->fetch_assoc()) {
        $sendDateRecords[$row['email_id']] = [
            'first_send_date' => $row['first_send_date'],
            'last_send_date' => $row['last_send_date']
        ];
    }
}

// Display results in a table format
logCustomMessage("📅 <strong>Email Send Dates:</strong>");
if (!empty($sendDateRecords)) {
    echo "<table border='1'><tr><th>Email ID</th><th>First Sent</th><th>Last Sent</th></tr>";
    foreach ($sendDateRecords as $emailId => $dates) {
        echo "<tr><td>{$emailId}</td><td>{$dates['first_send_date']}</td><td>{$dates['last_send_date']}</td></tr>\n";
    }
    echo "</table>";
} else {
    logCustomMessage("⚠️ No send date records found.");
}

// -----------------------------------------------
// Step 7: Insert first_send_date and last_send_date into click_date_and_total_sales_report
// -----------------------------------------------
logCustomMessage("<strong>Step 7:</strong> Updating first_send_date and last_send_date in click_date_and_total_sales_report");

$updatedDateRows = 0;

foreach ($sendDateRecords as $emailId => $dates) {
    $sqlUpdateSendDates = "UPDATE click_date_and_total_sales_report 
                           SET first_send_date = ?, last_send_date = ?
                           WHERE email_id = ?";

    $stmt = $conn->prepare($sqlUpdateSendDates);
    $stmt->bind_param("ssi", $dates['first_send_date'], $dates['last_send_date'], $emailId);
    
    if ($stmt->execute()) {
        $updatedDateRows += $stmt->affected_rows;
    }
    
    $stmt->close();
}

logCustomMessage("✅ first_send_date and last_send_date updated successfully. Rows affected: " . $updatedDateRows);

// -----------------------------------------------
// Step 8: Update status_stage_type based on latest_order_status
// -----------------------------------------------
logCustomMessage("<strong>Step 8:</strong> Updating status_stage_type based on latest_order_status");

$statusMapping = [
    // ✅ Completed or Successfully Processed Orders
    "Delivered" => "Completed or Successfully Processed Orders",
    "Ready for Pickup" => "Completed or Successfully Processed Orders",
    "Shipped" => "Completed or Successfully Processed Orders",
    "At Hub" => "Completed or Successfully Processed Orders",
    
    // 🔄 Orders In Process but Likely Paid
    "Ordered" => "Orders In Process but Likely Paid",
    "Processing" => "Orders In Process but Likely Paid",
    "Scheduled" => "Orders In Process but Likely Paid",
    "Shipped, Need Payment" => "Orders In Process but Likely Paid",
    "Scheduled, Need Payment" => "Orders In Process but Likely Paid",
    
    // 🚩 Uncertain, Pending, or Cancelled Orders
    "Cancelled" => "Uncertain, Pending, or Cancelled Orders",
    "Need To Cancel" => "Uncertain, Pending, or Cancelled Orders",
    "Chargeback" => "Uncertain, Pending, or Cancelled Orders",
    "Delivered - Open Issues" => "Uncertain, Pending, or Cancelled Orders",
    "Cust. Has Balance Due" => "Uncertain, Pending, or Cancelled Orders",
    "Delivered, Need Payment" => "Uncertain, Pending, or Cancelled Orders",
    "Customer Issue" => "Uncertain, Pending, or Cancelled Orders",
    "Syn-delivered open issue" => "Uncertain, Pending, or Cancelled Orders",
    "Backordered" => "Uncertain, Pending, or Cancelled Orders",
    "Syn-Backordered" => "Uncertain, Pending, or Cancelled Orders",
    "Pending Inventory" => "Uncertain, Pending, or Cancelled Orders",
    "Reauth. Required" => "Uncertain, Pending, or Cancelled Orders",
    "Pending Cust. Info" => "Uncertain, Pending, or Cancelled Orders",
    "Order On Hold" => "Uncertain, Pending, or Cancelled Orders",
    "Pending Check Clear" => "Uncertain, Pending, or Cancelled Orders"
];

$updatedStatusRows = 0;
foreach ($statusMapping as $status => $stageType) {
    $sqlUpdateStatus = "UPDATE click_date_and_total_sales_report 
                         SET status_stage_type = ? 
                         WHERE latest_order_status = ?";
    $stmt = $conn->prepare($sqlUpdateStatus);
    $stmt->bind_param("ss", $stageType, $status);
    if ($stmt->execute()) {
        $updatedStatusRows += $stmt->affected_rows;
    }
    $stmt->close();
}

logCustomMessage("✅ status_stage_type updated successfully. Rows affected: " . $updatedStatusRows);

// -----------------------------------------------
// Step 9: Clear existing data before inserting new funnel metrics
// -----------------------------------------------
logCustomMessage("<strong>Step 9:</strong> Clearing previous data from email_sales_funnel_report_transposed");

$sqlTruncate = "TRUNCATE TABLE email_sales_funnel_report_transposed";
if ($conn->query($sqlTruncate) === TRUE) {
    logCustomMessage("✅ Table email_sales_funnel_report_transposed has been truncated.");
} else {
    logCustomMessage("❌ SQL Error truncating table: " . $conn->error);
}

// -----------------------------------------------
// Step 9.1: Insert computed funnel metrics into email_sales_funnel_report_transposed
// -----------------------------------------------
logCustomMessage("<strong>Step 9.1:</strong> Inserting computed funnel metrics into email_sales_funnel_report_transposed");

$sqlInsertFunnelData = "
    INSERT INTO email_sales_funnel_report_transposed 
    (email_id, email_name, campaign_id, campaign_name, first_send_date, last_send_date, latest_order_status, status_stage_type, funnel_stage, value)
    
    WITH combinations AS (
        SELECT DISTINCT 
               main.email_id, 
               main.campaign_id, 
               /* Usamos MIN y MAX de first_send_date para que coincida con sales_per_lead. */
               COALESCE(
                   (SELECT MIN(sub.first_send_date)
                      FROM click_date_and_total_sales_report AS sub
                     WHERE sub.email_id    = main.email_id 
                       AND sub.campaign_id = main.campaign_id
                   ), 
                   '1969-12-31'
               ) AS first_send_date,
               COALESCE(
                   (SELECT MAX(sub.first_send_date)
                      FROM click_date_and_total_sales_report AS sub
                     WHERE sub.email_id    = main.email_id 
                       AND sub.campaign_id = main.campaign_id
                   ), 
                   '1969-12-31'
               ) AS last_send_date,
               
               /* Normalizamos latest_order_status a 'Unknown' si viene NULL */
               COALESCE(main.latest_order_status, 'Unknown') AS latest_order_status,
               main.status_stage_type
        FROM click_date_and_total_sales_report AS main
    ),
    
    sales_per_lead AS (
        SELECT 
            lead_id, 
            email_id, 
            campaign_id, 
            first_send_date,
            /* Si hay NULL en latest_order_status, podrías hacer también COALESCE(...) aquí */
            latest_order_status,
            status_stage_type,
            MAX(total_sales_in_7_days) AS total_sales,
            MAX(latest_order_amount)   AS latest_sales
        FROM click_date_and_total_sales_report
        WHERE total_sales_in_7_days > 0
        GROUP BY 
            lead_id, 
            email_id, 
            campaign_id, 
            first_send_date, 
            latest_order_status, 
            status_stage_type
    )

    -- avg_sale_value_7_days_after_click
    SELECT 
           comb.email_id, 
           'Unknown' AS email_name,  
           comb.campaign_id, 
           'Unknown' AS campaign_name,  
           comb.first_send_date, 
           comb.last_send_date,  
           comb.latest_order_status, 
           comb.status_stage_type, 
           'avg_sale_value_7_days_after_click' AS funnel_stage, 
           COALESCE(SUM(sales_per_lead.total_sales) / NULLIF(COUNT(DISTINCT sales_per_lead.lead_id), 0), 0) AS value
    FROM combinations comb
    LEFT JOIN sales_per_lead
           ON sales_per_lead.email_id            = comb.email_id 
          AND sales_per_lead.campaign_id         = comb.campaign_id 
          AND sales_per_lead.first_send_date     = comb.first_send_date 
          AND sales_per_lead.latest_order_status = comb.latest_order_status 
          AND sales_per_lead.status_stage_type   = comb.status_stage_type
    GROUP BY 
           comb.email_id, 
           comb.campaign_id, 
           comb.first_send_date, 
           comb.last_send_date, 
           comb.latest_order_status, 
           comb.status_stage_type

    UNION ALL

    -- total_sale_value_7_days_after_click (nuevo)
    SELECT
           comb.email_id,
           'Unknown',
           comb.campaign_id,
           'Unknown',
           comb.first_send_date,
           comb.last_send_date,
           comb.latest_order_status,
           comb.status_stage_type,
           'total_sale_value_7_days_after_click',
           COALESCE(SUM(sales_per_lead.total_sales), 0)
    FROM combinations comb
    LEFT JOIN sales_per_lead
           ON sales_per_lead.email_id            = comb.email_id
          AND sales_per_lead.campaign_id         = comb.campaign_id
          AND sales_per_lead.first_send_date     = comb.first_send_date
          AND sales_per_lead.latest_order_status = comb.latest_order_status
          AND sales_per_lead.status_stage_type   = comb.status_stage_type
    GROUP BY
           comb.email_id,
           comb.campaign_id,
           comb.first_send_date,
           comb.last_send_date,
           comb.latest_order_status,
           comb.status_stage_type

    UNION ALL

    -- total_sales_7_days_after_click
    SELECT 
           comb.email_id, 
           'Unknown', 
           comb.campaign_id, 
           'Unknown', 
           comb.first_send_date, 
           comb.last_send_date, 
           comb.latest_order_status, 
           comb.status_stage_type, 
           'total_sales_7_days_after_click', 
           COALESCE(COUNT(DISTINCT sales_per_lead.lead_id), 0)
    FROM combinations comb
    LEFT JOIN sales_per_lead
           ON sales_per_lead.email_id            = comb.email_id 
          AND sales_per_lead.campaign_id         = comb.campaign_id 
          AND sales_per_lead.first_send_date     = comb.first_send_date 
          AND sales_per_lead.latest_order_status = comb.latest_order_status 
          AND sales_per_lead.status_stage_type   = comb.status_stage_type
    GROUP BY 
           comb.email_id, 
           comb.campaign_id, 
           comb.first_send_date, 
           comb.last_send_date, 
           comb.latest_order_status, 
           comb.status_stage_type

    UNION ALL

    -- total_unique_historical_sales_from_leads_that_clicked
    SELECT 
           comb.email_id, 
           'Unknown', 
           comb.campaign_id, 
           'Unknown', 
           comb.first_send_date, 
           comb.last_send_date, 
           comb.latest_order_status, 
           comb.status_stage_type, 
           'total_unique_historical_sales_from_leads_that_clicked', 
           COALESCE(SUM(sales_per_lead.latest_sales), 0)
    FROM combinations comb
    LEFT JOIN sales_per_lead
           ON sales_per_lead.email_id            = comb.email_id 
          AND sales_per_lead.campaign_id         = comb.campaign_id 
          AND sales_per_lead.first_send_date     = comb.first_send_date 
          AND sales_per_lead.latest_order_status = comb.latest_order_status 
          AND sales_per_lead.status_stage_type   = comb.status_stage_type
    GROUP BY 
           comb.email_id, 
           comb.campaign_id, 
           comb.first_send_date, 
           comb.last_send_date, 
           comb.latest_order_status, 
           comb.status_stage_type

    UNION ALL

    -- total_historical_sales_from_leads_that_clicked
    SELECT 
           comb.email_id, 
           'Unknown', 
           comb.campaign_id, 
           'Unknown', 
           comb.first_send_date, 
           comb.last_send_date, 
           comb.latest_order_status, 
           comb.status_stage_type, 
           'total_historical_sales_from_leads_that_clicked', 
           COALESCE(SUM(sales_per_lead.latest_sales) / NULLIF(COUNT(DISTINCT sales_per_lead.lead_id), 0), 0)
    FROM combinations comb
    LEFT JOIN sales_per_lead
           ON sales_per_lead.email_id            = comb.email_id 
          AND sales_per_lead.campaign_id         = comb.campaign_id 
          AND sales_per_lead.first_send_date     = comb.first_send_date 
          AND sales_per_lead.latest_order_status = comb.latest_order_status 
          AND sales_per_lead.status_stage_type   = comb.status_stage_type
    GROUP BY 
           comb.email_id, 
           comb.campaign_id, 
           comb.first_send_date, 
           comb.last_send_date, 
           comb.latest_order_status, 
           comb.status_stage_type
";

if ($conn->query($sqlInsertFunnelData) === TRUE) {
    logCustomMessage("✅ Step 9.1: Data successfully inserted into email_sales_funnel_report_transposed.");
} else {
    logCustomMessage("❌ Step 9.1: SQL Error inserting funnel data: " . $conn->error);
}

// -----------------------------------------------
// Step 9.2: Insert general email metrics (solo anclado en email_id)
// -----------------------------------------------
logCustomMessage("<strong>Step 9.2:</strong> Inserting total_sent, total_opens, total_clicks, total_unique_clickers, total_bounces (by email_id)");

// -- A) Insert de 'total_sent'
$sqlInsertTotalSent = "
    INSERT INTO email_sales_funnel_report_transposed (
        email_id,
        email_name,
        campaign_id,
        campaign_name,
        first_send_date,
        last_send_date,
        latest_order_status,
        status_stage_type,
        funnel_stage,
        value
    )
    SELECT
        ec.email_id,
        ec.email_name,
        ec.campaign_id,
        ec.campaign_name,
        ec.first_send_date,
        ec.last_send_date,
        ec.latest_order_status,
        ec.status_stage_type,
        'total_sent' AS funnel_stage,
        COALESCE(bbe.total_sent, 0) AS value
    FROM (
        SELECT DISTINCT
            email_id,
            email_name,
            campaign_id,
            campaign_name,
            first_send_date,
            last_send_date,
            latest_order_status,
            status_stage_type
        FROM email_sales_funnel_report_transposed
    ) ec
    LEFT JOIN (
        SELECT 
            email_id,
            SUM(total_sent) AS total_sent
        FROM email_sales_funnel_base_dataset
        GROUP BY email_id
    ) bbe
      ON ec.email_id = bbe.email_id
";
if ($conn->query($sqlInsertTotalSent) === TRUE) {
    logCustomMessage("✅ Step 9.2A: total_sent inserted.");
} else {
    logCustomMessage("❌ Step 9.2A: SQL Error: " . $conn->error);
}


// -- B) Insert de 'total_opens'
$sqlInsertTotalOpens = "
    INSERT INTO email_sales_funnel_report_transposed (
        email_id,
        email_name,
        campaign_id,
        campaign_name,
        first_send_date,
        last_send_date,
        latest_order_status,
        status_stage_type,
        funnel_stage,
        value
    )
    SELECT
        ec.email_id,
        ec.email_name,
        ec.campaign_id,
        ec.campaign_name,
        ec.first_send_date,
        ec.last_send_date,
        ec.latest_order_status,
        ec.status_stage_type,
        'total_opens' AS funnel_stage,
        COALESCE(bbe.total_opens, 0) AS value
    FROM (
        SELECT DISTINCT
            email_id,
            email_name,
            campaign_id,
            campaign_name,
            first_send_date,
            last_send_date,
            latest_order_status,
            status_stage_type
        FROM email_sales_funnel_report_transposed
    ) ec
    LEFT JOIN (
        SELECT 
            email_id,
            SUM(total_opens) AS total_opens
        FROM email_sales_funnel_base_dataset
        GROUP BY email_id
    ) bbe
      ON ec.email_id = bbe.email_id
";
if ($conn->query($sqlInsertTotalOpens) === TRUE) {
    logCustomMessage("✅ Step 9.2B: total_opens inserted.");
} else {
    logCustomMessage("❌ Step 9.2B: SQL Error: " . $conn->error);
}


// -- C) Insert de 'total_clicks'
$sqlInsertTotalClicks = "
    INSERT INTO email_sales_funnel_report_transposed (
        email_id,
        email_name,
        campaign_id,
        campaign_name,
        first_send_date,
        last_send_date,
        latest_order_status,
        status_stage_type,
        funnel_stage,
        value
    )
    SELECT
        ec.email_id,
        ec.email_name,
        ec.campaign_id,
        ec.campaign_name,
        ec.first_send_date,
        ec.last_send_date,
        ec.latest_order_status,
        ec.status_stage_type,
        'total_clicks' AS funnel_stage,
        COALESCE(bbe.total_clicks, 0) AS value
    FROM (
        SELECT DISTINCT
            email_id,
            email_name,
            campaign_id,
            campaign_name,
            first_send_date,
            last_send_date,
            latest_order_status,
            status_stage_type
        FROM email_sales_funnel_report_transposed
    ) ec
    LEFT JOIN (
        SELECT 
            email_id,
            SUM(total_clicks) AS total_clicks
        FROM email_sales_funnel_base_dataset
        GROUP BY email_id
    ) bbe
      ON ec.email_id = bbe.email_id
";
if ($conn->query($sqlInsertTotalClicks) === TRUE) {
    logCustomMessage("✅ Step 9.2C: total_clicks inserted.");
} else {
    logCustomMessage("❌ Step 9.2C: SQL Error: " . $conn->error);
}


// -- D) Insert de 'total_unique_clickers'
$sqlInsertTotalUniqueClickers = "
    INSERT INTO email_sales_funnel_report_transposed (
        email_id,
        email_name,
        campaign_id,
        campaign_name,
        first_send_date,
        last_send_date,
        latest_order_status,
        status_stage_type,
        funnel_stage,
        value
    )
    SELECT
        ec.email_id,
        ec.email_name,
        ec.campaign_id,
        ec.campaign_name,
        ec.first_send_date,
        ec.last_send_date,
        ec.latest_order_status,
        ec.status_stage_type,
        'total_unique_clickers' AS funnel_stage,
        COALESCE(bbe.total_unique_clickers, 0) AS value
    FROM (
        SELECT DISTINCT
            email_id,
            email_name,
            campaign_id,
            campaign_name,
            first_send_date,
            last_send_date,
            latest_order_status,
            status_stage_type
        FROM email_sales_funnel_report_transposed
    ) ec
    LEFT JOIN (
        SELECT 
            email_id,
            SUM(total_unique_clickers) AS total_unique_clickers
        FROM email_sales_funnel_base_dataset
        GROUP BY email_id
    ) bbe
      ON ec.email_id = bbe.email_id
";
if ($conn->query($sqlInsertTotalUniqueClickers) === TRUE) {
    logCustomMessage("✅ Step 9.2D: total_unique_clickers inserted.");
} else {
    logCustomMessage("❌ Step 9.2D: SQL Error: " . $conn->error);
}


// -- E) Insert de 'total_bounces'
$sqlInsertTotalBounces = "
    INSERT INTO email_sales_funnel_report_transposed (
        email_id,
        email_name,
        campaign_id,
        campaign_name,
        first_send_date,
        last_send_date,
        latest_order_status,
        status_stage_type,
        funnel_stage,
        value
    )
    SELECT
        ec.email_id,
        ec.email_name,
        ec.campaign_id,
        ec.campaign_name,
        ec.first_send_date,
        ec.last_send_date,
        ec.latest_order_status,
        ec.status_stage_type,
        'total_bounces' AS funnel_stage,
        COALESCE(bbe.total_bounces, 0) AS value
    FROM (
        SELECT DISTINCT
            email_id,
            email_name,
            campaign_id,
            campaign_name,
            first_send_date,
            last_send_date,
            latest_order_status,
            status_stage_type
        FROM email_sales_funnel_report_transposed
    ) ec
    LEFT JOIN (
        SELECT 
            email_id,
            SUM(total_bounces) AS total_bounces
        FROM email_sales_funnel_base_dataset
        GROUP BY email_id
    ) bbe
      ON ec.email_id = bbe.email_id
";
if ($conn->query($sqlInsertTotalBounces) === TRUE) {
    logCustomMessage("✅ Step 9.2E: total_bounces inserted.");
} else {
    logCustomMessage("❌ Step 9.2E: SQL Error: " . $conn->error);
}


// -----------------------------------------------
// Step 10: Update email_name and campaign_name
// -----------------------------------------------
logCustomMessage("<strong>Step 10:</strong> Updating email_name and campaign_name from emma_mason_email_campaign_stats.");

$sqlUpdateNames = "
    UPDATE email_sales_funnel_report_transposed esf
    JOIN emma_mason_email_campaign_stats emecs 
        ON esf.email_id = emecs.email_id
    SET esf.email_name = COALESCE(emecs.email_name, 'Unknown'),
        esf.campaign_name = COALESCE(emecs.category_title, 'Unknown')
    WHERE esf.email_name = 'Unknown' OR esf.campaign_name = 'Unknown';
";

if ($conn->query($sqlUpdateNames) === TRUE) {
    logCustomMessage("✅ Step 10: email_name and campaign_name successfully updated.");
} else {
    logCustomMessage("❌ Step 10: SQL Error updating names: " . $conn->error);
}


// -----------------------------------------------
// Close Database Connection
// -----------------------------------------------
$conn->close();
logCustomMessage("✅ <strong>Process completed successfully.</strong>");
?>