<?php
/**
 * Populates the email_sales_funnel_report table with aggregated email sales funnel data.
 */

require_once __DIR__ . '/../config.php';
require_once __DIR__ . '/utils.php';

// -----------------------------------------------
// Database Connection
// -----------------------------------------------
$conn = new mysqli(DB_HOST, DB_USER, DB_PASSWORD, DB_NAME);
if ($conn->connect_error) {
    die("<p style='color:red;'>❌ Database connection error: " . $conn->connect_error . "</p>");
}
$conn->set_charset("utf8mb4");

// Function to log messages
function logCustomMessage($message) {
    echo "<p>" . htmlspecialchars($message) . "</p>\n";
}

// -----------------------------------------------
// Step 1: Truncate email_sales_funnel_report
// -----------------------------------------------
function truncateFunnelTable($conn) {
    logCustomMessage("<strong>Step 1:</strong> Truncating email_sales_funnel_report");
    $sql = "TRUNCATE TABLE email_sales_funnel_report";
    return $conn->query($sql) ? logCustomMessage("✅ Table truncated successfully.") 
                              : logCustomMessage("❌ SQL Error truncating table: " . $conn->error);
}

// -----------------------------------------------
// Step 2: Update parsed_click_data from mautic_leads
// -----------------------------------------------
function updateParsedClickData($conn) {
    logCustomMessage("<strong>Step 2:</strong> Updating parsed_click_data with latest_order_status and status_stage_type");

    $sql = "
    UPDATE ssgenius_emma_mason_mautic_data_warehouse.parsed_click_data p
    LEFT JOIN ssgenius_emma_mason_mautic_data_warehouse.mautic_leads ml ON p.lead_id = ml.id
    SET 
        p.latest_order_status = ml.latest_order_status,
        p.status_stage_type = CASE
            WHEN ml.latest_order_status IN ('Delivered', 'Ready for Pickup', 'Shipped', 'At Hub') THEN 'Completed or Successfully Processed Orders'
            WHEN ml.latest_order_status IN ('Ordered','Processing','Scheduled','Shipped, Need Payment','Scheduled, Need Payment') THEN 'Orders In Process but Likely Paid'
            WHEN ml.latest_order_status IN ('Cancelled','Need To Cancel','Chargeback','Delivered - Open Issues','Cust. Has Balance Due','Delivered, Need Payment',
                                            'Customer Issue','Syn-delivered open issue','Backordered','Syn-Backordered','Pending Inventory',
                                            'Reauth. Required','Pending Cust. Info','Order On Hold','Pending Check Clear') 
                THEN 'Uncertain, Pending, or Cancelled Orders'
            ELSE 'Unknown'
        END";

    return $conn->query($sql) ? logCustomMessage("✅ parsed_click_data updated successfully.") 
                              : logCustomMessage("❌ SQL Error updating parsed_click_data: " . $conn->error);
}

// -----------------------------------------------
// Step 3: Insert funnel data
// -----------------------------------------------
function insertFunnelData($conn) {
    logCustomMessage("<strong>Step 3:</strong> Inserting data by status into email_sales_funnel_report");

    $sql = "
    INSERT INTO email_sales_funnel_report (email_id, latest_order_status, total_users_per_status)
    SELECT 
        t1.email_id, t2.latest_order_status, COALESCE(t3.total_count, 0)
    FROM
        (SELECT DISTINCT email_id FROM parsed_click_data WHERE email_id IS NOT NULL AND email_id <> '') AS t1
    CROSS JOIN 
        (SELECT DISTINCT latest_order_status FROM parsed_click_data WHERE latest_order_status IS NOT NULL AND latest_order_status <> '') AS t2
    LEFT JOIN 
        (SELECT email_id, latest_order_status, COUNT(*) AS total_count FROM parsed_click_data GROUP BY email_id, latest_order_status) AS t3
    ON t3.email_id = t1.email_id AND t3.latest_order_status = t2.latest_order_status";

    return $conn->query($sql) ? logCustomMessage("✅ Data inserted successfully into email_sales_funnel_report.") 
                              : logCustomMessage("❌ SQL Error inserting funnel data: " . $conn->error);
}

// -----------------------------------------------
// Step 4: Update totals from email_activity_per_day (WITHOUT total_clicks)
// -----------------------------------------------
function updateActivityData($conn) {
    logCustomMessage("<strong>Step 4:</strong> Updating email_sales_funnel_report without total_clicks");

    $sql = "
    UPDATE email_sales_funnel_report esf
    JOIN (
        SELECT email_id, MAX(campaign_id) AS campaign_id, 
               SUM(total_sent) AS total_sent, SUM(total_opens) AS total_opens,
               MIN(calendar_date) AS first_send_date, MAX(calendar_date) AS last_send_date
        FROM email_activity_per_day 
        GROUP BY email_id
    ) ea ON esf.email_id = ea.email_id
    SET esf.campaign_id = ea.campaign_id, esf.total_sent = ea.total_sent,
        esf.total_opens = ea.total_opens, esf.first_send_date = ea.first_send_date,
        esf.last_send_date = ea.last_send_date;
    ";

    return $conn->query($sql) ? logCustomMessage("✅ Successfully updated email_sales_funnel_report without total_clicks.") 
                              : logCustomMessage("❌ SQL Error updating email activity data: " . $conn->error);
}

// -----------------------------------------------
// Step 5: Fetch total_bounces from mautic_bounce_by_webhook_events
// -----------------------------------------------
function updateBounces($conn) {
    logCustomMessage("<strong>Step 5:</strong> Updating email_sales_funnel_report with bounces data");

    $sql = "
    UPDATE email_sales_funnel_report esf
    LEFT JOIN (
        SELECT email_id, COUNT(*) AS total_bounces FROM mautic_bounce_by_webhook_events GROUP BY email_id
    ) mbw ON esf.email_id = mbw.email_id
    SET esf.total_bounces = COALESCE(mbw.total_bounces, 0)";

    return $conn->query($sql) ? logCustomMessage("✅ Successfully updated email_sales_funnel_report with bounce data.") 
                              : logCustomMessage("❌ SQL Error updating bounces: " . $conn->error);
}

// -----------------------------------------------
// Step 6: Fetch email_name and campaign_name
// -----------------------------------------------
function updateNames($conn) {
    logCustomMessage("<strong>Step 6:</strong> Updating email_sales_funnel_report with email_name and campaign_name");

    $sql = "
    UPDATE email_sales_funnel_report esf
    LEFT JOIN emma_mason_email_details emed ON esf.email_id = emed.email_id
    LEFT JOIN emma_mason_campaign_details emcd ON esf.campaign_id = emcd.campaign_id
    SET 
        esf.email_name = COALESCE(emed.name, ''),  -- ✅ Ensure email_name is never NULL
        esf.campaign_name = COALESCE(emcd.name, '');  -- ✅ Ensure campaign_name is never NULL
    ";

    return $conn->query($sql) ? logCustomMessage("✅ Successfully updated email_sales_funnel_report with email_name and campaign_name.") 
                              : logCustomMessage("❌ SQL Error updating names: " . $conn->error);
}

// -----------------------------------------------
// Step 7: Fetch total_clicks and total_unique_clickers from parsed_click_data
// -----------------------------------------------
function updateClicksData($conn) {
    logCustomMessage("<strong>Step 7:</strong> Updating email_sales_funnel_report with total_clicks and total_unique_clickers");

    $sql = "
    UPDATE email_sales_funnel_report esf
    LEFT JOIN (
        SELECT email_id, latest_order_status, 
               COUNT(*) AS total_clicks,  -- ✅ Counts ALL click events
               COUNT(DISTINCT lead_id) AS total_unique_clickers  -- ✅ Counts UNIQUE lead clicks
        FROM parsed_click_data
        WHERE lead_id IS NOT NULL  -- Ignore NULL leads
        GROUP BY email_id, latest_order_status
    ) pcd ON esf.email_id = pcd.email_id AND esf.latest_order_status = pcd.latest_order_status
    SET esf.total_clicks = COALESCE(pcd.total_clicks, 0), 
        esf.total_unique_clickers = COALESCE(pcd.total_unique_clickers, 0);
    ";

    return $conn->query($sql) ? logCustomMessage("✅ Successfully updated total_clicks and total_unique_clickers.") 
                              : logCustomMessage("❌ SQL Error updating clicks data: " . $conn->error);
}

// -----------------------------------------------
// Step 8: Fetch total sales and transaction counts for users who received or opened the email
// -----------------------------------------------
function updateSalesForReceivedAndOpenedEmails($conn) {
    logCustomMessage("<strong>Step 8:</strong> Updating email_sales_funnel_report with total sales for received/opened emails");

    $sql = "
    UPDATE email_sales_funnel_report esf
    LEFT JOIN (
        SELECT 
            mse.email_id,
            ml.latest_order_status,
            
            -- Total sales for users that RECEIVED the email
            SUM(ml.latest_order_amount) AS total_sales_for_users_that_received_the_email,
            COUNT(*) AS sales_transaction_count_all_users_that_received_the_email,
            
            -- Total sales for users that OPENED the email
            SUM(CASE WHEN mse.date_read IS NOT NULL AND mse.date_read > 0 THEN ml.latest_order_amount ELSE 0 END) 
                AS total_sales_for_users_that_opened_the_email,
            COUNT(CASE WHEN mse.date_read IS NOT NULL AND mse.date_read > 0 THEN ml.id ELSE NULL END) 
                AS sales_transaction_count_all_users_that_opened_the_email

        FROM mautic_stats_email_full mse
        JOIN mautic_leads ml ON mse.lead_id = ml.id  -- ✅ FIXED
        WHERE mse.date_sent IS NOT NULL AND mse.date_sent > 0  -- Users who RECEIVED the email
        GROUP BY mse.email_id, ml.latest_order_status
    ) sales_data 
    ON esf.email_id = sales_data.email_id AND esf.latest_order_status = sales_data.latest_order_status
    SET 
        esf.total_sales_for_users_that_received_the_email = COALESCE(sales_data.total_sales_for_users_that_received_the_email, 0),
        esf.sales_transaction_count_all_users_that_received_the_email = COALESCE(sales_data.sales_transaction_count_all_users_that_received_the_email, 0),
        esf.total_sales_for_users_that_opened_the_email = COALESCE(sales_data.total_sales_for_users_that_opened_the_email, 0),
        esf.sales_transaction_count_all_users_that_opened_the_email = COALESCE(sales_data.sales_transaction_count_all_users_that_opened_the_email, 0);
    ";

    return $conn->query($sql) ? logCustomMessage("✅ Successfully updated total sales for received/opened emails.") 
                              : logCustomMessage("❌ SQL Error updating sales for received/opened emails: " . $conn->error);
}

// -----------------------------------------------
// Step 9: Fetch sales transactions and total sales 7 days after a click (grouped by latest_order_status)
// -----------------------------------------------
function updateSales7DaysAfterClick($conn) {
    logCustomMessage("<strong>Step 9:</strong> Updating email_sales_funnel_report with sales data 7 days after click, grouped by latest_order_status");

    $sql = "
    UPDATE email_sales_funnel_report esf
    LEFT JOIN (
        WITH ClickedSales AS (
            SELECT 
                pcd.email_id,
                ml.latest_order_status,
                COUNT(ml.id) AS sales_transactions_count_7_days_after_email,
                SUM(ml.latest_order_amount) AS total_sales_7_days_after_click
            FROM parsed_click_data pcd
            JOIN mautic_leads ml ON pcd.lead_id = ml.id
            WHERE ml.latest_order_date IS NOT NULL 
                AND DATE(ml.latest_order_date) BETWEEN DATE(pcd.click_date) AND DATE(pcd.click_date) + INTERVAL 7 DAY
            GROUP BY pcd.email_id, ml.latest_order_status
        )
        SELECT * FROM ClickedSales
    ) sales_data 
    ON esf.email_id = sales_data.email_id AND esf.latest_order_status = sales_data.latest_order_status
    SET 
        esf.sales_transactions_count_7_days_after_email = COALESCE(sales_data.sales_transactions_count_7_days_after_email, 0),
        esf.total_sales_7_days_after_click = COALESCE(sales_data.total_sales_7_days_after_click, 0);
    ";

    return $conn->query($sql) ? logCustomMessage("✅ Successfully updated sales transactions and total sales 7 days after click, grouped by latest_order_status.") 
                              : logCustomMessage("❌ SQL Error updating sales 7 days after click: " . $conn->error);
}

// -----------------------------------------------
// Step 10: Update all three average sale values
// -----------------------------------------------
function updateAverageSaleValues($conn) {
    logCustomMessage("<strong>Step 10:</strong> Updating email_sales_funnel_report with all average sale values");

    $sql = "
    UPDATE email_sales_funnel_report
    SET 
        avg_sale_value_7_days_after_click = 
            CASE 
                WHEN sales_transactions_count_7_days_after_email > 0 
                THEN total_sales_7_days_after_click / sales_transactions_count_7_days_after_email
                ELSE 0 
            END,

        avg_sale_value_all_users_that_received_the_email = 
            CASE 
                WHEN sales_transaction_count_all_users_that_received_the_email > 0 
                THEN total_sales_for_users_that_received_the_email / sales_transaction_count_all_users_that_received_the_email
                ELSE 0 
            END,

        avg_sale_value_all_users_that_opened_the_email = 
            CASE 
                WHEN sales_transaction_count_all_users_that_opened_the_email > 0 
                THEN total_sales_for_users_that_opened_the_email / sales_transaction_count_all_users_that_opened_the_email
                ELSE 0 
            END
    WHERE 
        total_sales_7_days_after_click IS NOT NULL 
        OR total_sales_for_users_that_received_the_email IS NOT NULL 
        OR total_sales_for_users_that_opened_the_email IS NOT NULL;
    ";

    return $conn->query($sql) ? logCustomMessage("✅ Successfully updated all average sale values.") 
                              : logCustomMessage("❌ SQL Error updating average sale values: " . $conn->error);
}

// -----------------------------------------------
// Step 11: Update status_stage_type in email_sales_funnel_report
// -----------------------------------------------
function updateStatusStageType($conn) {
    logCustomMessage("<strong>Step 11:</strong> Updating email_sales_funnel_report with status_stage_type");

    $sql = "
    UPDATE email_sales_funnel_report
    SET status_stage_type = 
        CASE 
            -- ✅ Completed or Successfully Processed Orders
            WHEN latest_order_status IN ('Delivered', 'Ready for Pickup', 'Shipped', 'At Hub') 
            THEN 'Completed or Successfully Processed Orders'

            -- 🔄 Orders In Process but Likely Paid
            WHEN latest_order_status IN ('Ordered', 'Processing', 'Scheduled', 'Shipped, Need Payment', 'Scheduled, Need Payment') 
            THEN 'Orders In Process but Likely Paid'

            -- 🚩 Uncertain, Pending, or Cancelled Orders
            WHEN latest_order_status IN ('Cancelled', 'Need To Cancel', 'Chargeback', 'Delivered - Open Issues', 'Cust. Has Balance Due', 
                                         'Delivered, Need Payment', 'Customer Issue', 'Syn-delivered open issue', 'Backordered', 
                                         'Syn-Backordered', 'Pending Inventory', 'Reauth. Required', 'Pending Cust. Info', 
                                         'Order On Hold', 'Pending Check Clear') 
            THEN 'Uncertain, Pending, or Cancelled Orders'

            -- ❓ Default case (unknown statuses)
            ELSE 'Unknown'
        END;
    ";

    return $conn->query($sql) ? logCustomMessage("✅ Successfully updated status_stage_type in email_sales_funnel_report.") 
                              : logCustomMessage("❌ SQL Error updating status_stage_type: " . $conn->error);
}

// -----------------------------------------------
// Step 12: Truncate email_sales_funnel_report_transposed
// -----------------------------------------------
function truncateFunnelTableTransposed($conn) {
    logCustomMessage("<strong>Step 12:</strong> Truncating email_sales_funnel_report_transposed");
    $sql = "TRUNCATE TABLE email_sales_funnel_report_transposed";
    return $conn->query($sql) ? logCustomMessage("✅ Table truncated successfully.") 
                              : logCustomMessage("❌ SQL Error truncating table: " . $conn->error);
}

// -----------------------------------------------
// Step 13: Insert All Possible Lines
// -----------------------------------------------
function insertFunnelRows($conn) {
    logCustomMessage("<strong>Step 13:</strong> Creating all possible rows for email_sales_funnel_report_transposed");

    $sql = "
    INSERT INTO email_sales_funnel_report_transposed 
    (email_id, email_name, campaign_id, campaign_name, first_send_date, last_send_date, latest_order_status, status_stage_type, funnel_stage, value)
    SELECT 
        esf.email_id, 
        esf.email_name, 
        esf.campaign_id, 
        esf.campaign_name, 
        COALESCE(esf.first_send_date, '1970-01-01') AS first_send_date, 
        COALESCE(esf.last_send_date, '1970-01-01') AS last_send_date, 
        esf.latest_order_status, 
        esf.status_stage_type, 
        funnel_stages.funnel_stage, 
        0 AS value -- Default value, will be updated later
    FROM email_sales_funnel_report esf
    CROSS JOIN (
        SELECT 'total_sent' AS funnel_stage UNION ALL
        SELECT 'total_opens' UNION ALL
        SELECT 'total_clicks' UNION ALL
        SELECT 'total_unique_clickers' UNION ALL
        SELECT 'total_bounces' UNION ALL
        SELECT 'total_sales_7_days_after_click' UNION ALL
        SELECT 'total_sales_for_users_that_received_the_email' UNION ALL
        SELECT 'total_sales_for_users_that_opened_the_email' UNION ALL
        SELECT 'sales_transactions_count_7_days_after_email' UNION ALL
        SELECT 'sales_transaction_count_all_users_that_received_the_email' UNION ALL
        SELECT 'sales_transaction_count_all_users_that_opened_the_email' UNION ALL
        SELECT 'avg_sale_value_7_days_after_click' UNION ALL
        SELECT 'avg_sale_value_all_users_that_received_the_email' UNION ALL
        SELECT 'avg_sale_value_all_users_that_opened_the_email' UNION ALL
        SELECT 'total_users_per_status'
    ) AS funnel_stages;
    ";

    return $conn->query($sql) ? logCustomMessage("✅ Rows inserted successfully into email_sales_funnel_report_transposed.") 
                              : logCustomMessage("❌ SQL Error inserting rows: " . $conn->error);
}

// -----------------------------------------------
// Step 14: Populate Values into email_sales_funnel_report_transposed
// -----------------------------------------------
function populateFunnelValues($conn) {
    logCustomMessage("<strong>Step 14:</strong> Populating values in email_sales_funnel_report_transposed");

    $updateQueries = [
        "total_sent" => "UPDATE email_sales_funnel_report_transposed eft 
            JOIN email_sales_funnel_report esf 
            ON eft.email_id = esf.email_id AND eft.latest_order_status = esf.latest_order_status 
            SET eft.value = esf.total_sent 
            WHERE eft.funnel_stage = 'total_sent';",

        "total_opens" => "UPDATE email_sales_funnel_report_transposed eft 
            JOIN email_sales_funnel_report esf 
            ON eft.email_id = esf.email_id AND eft.latest_order_status = esf.latest_order_status 
            SET eft.value = esf.total_opens 
            WHERE eft.funnel_stage = 'total_opens';",

        "total_clicks" => "UPDATE email_sales_funnel_report_transposed eft 
            JOIN email_sales_funnel_report esf 
            ON eft.email_id = esf.email_id AND eft.latest_order_status = esf.latest_order_status 
            SET eft.value = esf.total_clicks 
            WHERE eft.funnel_stage = 'total_clicks';",

        "total_unique_clickers" => "UPDATE email_sales_funnel_report_transposed eft 
            JOIN email_sales_funnel_report esf 
            ON eft.email_id = esf.email_id AND eft.latest_order_status = esf.latest_order_status 
            SET eft.value = esf.total_unique_clickers 
            WHERE eft.funnel_stage = 'total_unique_clickers';",

        "total_bounces" => "UPDATE email_sales_funnel_report_transposed eft 
            JOIN email_sales_funnel_report esf 
            ON eft.email_id = esf.email_id AND eft.latest_order_status = esf.latest_order_status 
            SET eft.value = esf.total_bounces 
            WHERE eft.funnel_stage = 'total_bounces';",

        "total_sales_7_days_after_click" => "UPDATE email_sales_funnel_report_transposed eft 
            JOIN email_sales_funnel_report esf 
            ON eft.email_id = esf.email_id AND eft.latest_order_status = esf.latest_order_status 
            SET eft.value = esf.total_sales_7_days_after_click 
            WHERE eft.funnel_stage = 'total_sales_7_days_after_click';",

        "total_sales_for_users_that_received_the_email" => "UPDATE email_sales_funnel_report_transposed eft 
            JOIN email_sales_funnel_report esf 
            ON eft.email_id = esf.email_id AND eft.latest_order_status = esf.latest_order_status 
            SET eft.value = esf.total_sales_for_users_that_received_the_email 
            WHERE eft.funnel_stage = 'total_sales_for_users_that_received_the_email';",

        "total_sales_for_users_that_opened_the_email" => "UPDATE email_sales_funnel_report_transposed eft 
            JOIN email_sales_funnel_report esf 
            ON eft.email_id = esf.email_id AND eft.latest_order_status = esf.latest_order_status 
            SET eft.value = esf.total_sales_for_users_that_opened_the_email 
            WHERE eft.funnel_stage = 'total_sales_for_users_that_opened_the_email';",

        "sales_transactions_count_7_days_after_email" => "UPDATE email_sales_funnel_report_transposed eft 
            JOIN email_sales_funnel_report esf 
            ON eft.email_id = esf.email_id AND eft.latest_order_status = esf.latest_order_status 
            SET eft.value = esf.sales_transactions_count_7_days_after_email 
            WHERE eft.funnel_stage = 'sales_transactions_count_7_days_after_email';",

        "sales_transaction_count_all_users_that_received_the_email" => "UPDATE email_sales_funnel_report_transposed eft 
            JOIN email_sales_funnel_report esf 
            ON eft.email_id = esf.email_id AND eft.latest_order_status = esf.latest_order_status 
            SET eft.value = esf.sales_transaction_count_all_users_that_received_the_email 
            WHERE eft.funnel_stage = 'sales_transaction_count_all_users_that_received_the_email';",

        "sales_transaction_count_all_users_that_opened_the_email" => "UPDATE email_sales_funnel_report_transposed eft 
            JOIN email_sales_funnel_report esf 
            ON eft.email_id = esf.email_id AND eft.latest_order_status = esf.latest_order_status 
            SET eft.value = esf.sales_transaction_count_all_users_that_opened_the_email 
            WHERE eft.funnel_stage = 'sales_transaction_count_all_users_that_opened_the_email';",

        "avg_sale_value_7_days_after_click" => "UPDATE email_sales_funnel_report_transposed eft 
            JOIN email_sales_funnel_report esf 
            ON eft.email_id = esf.email_id AND eft.latest_order_status = esf.latest_order_status 
            SET eft.value = esf.avg_sale_value_7_days_after_click 
            WHERE eft.funnel_stage = 'avg_sale_value_7_days_after_click';",

        "avg_sale_value_all_users_that_received_the_email" => "UPDATE email_sales_funnel_report_transposed eft 
            JOIN email_sales_funnel_report esf 
            ON eft.email_id = esf.email_id AND eft.latest_order_status = esf.latest_order_status 
            SET eft.value = esf.avg_sale_value_all_users_that_received_the_email 
            WHERE eft.funnel_stage = 'avg_sale_value_all_users_that_received_the_email';",

        "avg_sale_value_all_users_that_opened_the_email" => "UPDATE email_sales_funnel_report_transposed eft 
            JOIN email_sales_funnel_report esf 
            ON eft.email_id = esf.email_id AND eft.latest_order_status = esf.latest_order_status 
            SET eft.value = esf.avg_sale_value_all_users_that_opened_the_email 
            WHERE eft.funnel_stage = 'avg_sale_value_all_users_that_opened_the_email';",

        "total_users_per_status" => "UPDATE email_sales_funnel_report_transposed eft 
            SET eft.value = (SELECT COUNT(*) FROM email_sales_funnel_report esf 
                             WHERE esf.email_id = eft.email_id AND esf.latest_order_status = eft.latest_order_status) 
            WHERE eft.funnel_stage = 'total_users_per_status';"
    ];

    foreach ($updateQueries as $stage => $query) {
        if ($conn->query($query)) {
            logCustomMessage("✅ Successfully updated $stage.");
        } else {
            logCustomMessage("❌ SQL Error updating $stage: " . $conn->error);
        }
    }
}


// -----------------------------------------------
// EXECUTE ALL STEPS
// -----------------------------------------------
truncateFunnelTable($conn);
updateParsedClickData($conn);
insertFunnelData($conn);
updateActivityData($conn);
updateBounces($conn);
updateNames($conn);
updateClicksData($conn);
updateSalesForReceivedAndOpenedEmails($conn);
updateSales7DaysAfterClick($conn);
updateAverageSaleValues($conn);
updateStatusStageType($conn);
truncateFunnelTableTransposed($conn);
insertFunnelRows($conn);
populateFunnelValues($conn);


// Close connection
$conn->close();
logCustomMessage("✅ <strong>Process completed successfully.</strong>");
?>
