<?php
/**
 * Populates the email_sales_funnel_report_daily table with daily aggregated sales funnel data.
 */

require_once __DIR__ . '/../config.php';
require_once __DIR__ . '/utils.php';

// -----------------------------------------------
// Database Connection
// -----------------------------------------------
$conn = new mysqli(DB_HOST, DB_USER, DB_PASSWORD, DB_NAME);
if ($conn->connect_error) {
    die("<p style='color:red;'>❌ Database connection error: " . $conn->connect_error . "</p>");
}
$conn->set_charset("utf8mb4");

// Function to log messages
function logCustomMessage($message) {
    echo "<p>" . htmlspecialchars($message) . "</p>\n";
}

// -----------------------------------------------
// Step 1: Truncate the daily report table
// -----------------------------------------------
function truncateDailyReport($conn) {
    logCustomMessage("<strong>Step 1:</strong> Truncating email_sales_funnel_report_daily");
    $sql = "TRUNCATE TABLE email_sales_funnel_report_daily";
    return $conn->query($sql)
        ? logCustomMessage("✅ Table truncated successfully.") 
        : logCustomMessage("❌ SQL Error truncating table: " . $conn->error);
}

// -----------------------------------------------
// Step 2: Insert All Possible Rows
//  (Filter out NULL email_id to avoid insertion errors)
// -----------------------------------------------
function insertDailyFunnelRows($conn) {
    logCustomMessage("<strong>Step 2:</strong> Creating all possible rows for email_sales_funnel_report_daily");

    $sql = "
    INSERT INTO email_sales_funnel_report_daily
        (email_id, calendar_date, latest_order_status, total_sent, total_opens, total_clicks, status_stage_type)
    SELECT 
        emails.email_id,
        dates.calendar_date,
        statuses.latest_order_status,
        0 AS total_sent,
        0 AS total_opens,
        0 AS total_clicks,
        NULL AS status_stage_type
    FROM 
        (
            SELECT DISTINCT email_id 
            FROM email_activity_per_day
            WHERE email_id IS NOT NULL        /* ✅ Filter out NULL email_id */
        ) AS emails
    CROSS JOIN 
        (
            SELECT DISTINCT calendar_date
            FROM email_activity_per_day
        ) AS dates
    CROSS JOIN 
        (
            SELECT DISTINCT latest_order_status
            FROM email_sales_funnel_report
        ) AS statuses
    ";

    if ($conn->query($sql)) {
        logCustomMessage("✅ Rows inserted successfully into email_sales_funnel_report_daily.");
    } else {
        logCustomMessage("❌ SQL Error inserting rows: " . $conn->error);
    }
}

// -----------------------------------------------
// Step 3: Populate Basic Daily Values (No Calculations Needed)
// -----------------------------------------------
function populateBasicDailyFunnelValues($conn) {
    logCustomMessage("<strong>Step 3:</strong> Populating basic values in email_sales_funnel_report_daily");

    $updateQueries = [
        "total_sent" => "
            UPDATE email_sales_funnel_report_daily d
            JOIN email_activity_per_day a 
               ON d.email_id = a.email_id 
              AND d.calendar_date = a.calendar_date
            SET d.total_sent = a.total_sent;
        ",

        "total_opens" => "
            UPDATE email_sales_funnel_report_daily d
            JOIN email_activity_per_day a 
               ON d.email_id = a.email_id
              AND d.calendar_date = a.calendar_date
            SET d.total_opens = a.total_opens;
        ",

        "total_clicks" => "
            UPDATE email_sales_funnel_report_daily d
            JOIN email_activity_per_day a 
               ON d.email_id = a.email_id
              AND d.calendar_date = a.calendar_date
            SET d.total_clicks = a.total_clicks;
        "
    ];

    foreach ($updateQueries as $stage => $query) {
        if ($conn->query($query)) {
            logCustomMessage("✅ Successfully updated $stage.");
        } else {
            logCustomMessage("❌ SQL Error updating $stage: " . $conn->error);
        }
    }
}

// -----------------------------------------------
// Step 4: Populate status_stage_type
// -----------------------------------------------
function updateStatusStageType($conn) {
    logCustomMessage("<strong>Step 4:</strong> Updating status_stage_type in email_sales_funnel_report_daily");

    $sql = "
    UPDATE email_sales_funnel_report_daily
    SET status_stage_type = 
        CASE 
            WHEN latest_order_status IN (
                'Delivered', 'Ready for Pickup', 'Shipped', 'At Hub'
            ) THEN 'Completed or Successfully Processed Orders'

            WHEN latest_order_status IN (
                'Ordered', 'Processing', 'Scheduled', 
                'Shipped, Need Payment', 'Scheduled, Need Payment'
            ) THEN 'Orders In Process but Likely Paid'

            WHEN latest_order_status IN (
                'Cancelled','Need To Cancel','Chargeback','Delivered - Open Issues',
                'Cust. Has Balance Due','Delivered, Need Payment','Customer Issue',
                'Syn-delivered open issue','Backordered','Syn-Backordered','Pending Inventory',
                'Reauth. Required','Pending Cust. Info','Order On Hold','Pending Check Clear'
            ) THEN 'Uncertain, Pending, or Cancelled Orders'

            ELSE 'Unknown'
        END
    ";

    if ($conn->query($sql)) {
        logCustomMessage("✅ Successfully updated status_stage_type.");
    } else {
        logCustomMessage("❌ SQL Error updating status_stage_type: " . $conn->error);
    }
}

// -----------------------------------------------
// Step 5: Populate total_bounces
// -----------------------------------------------
function populateBounces($conn) {
    logCustomMessage("<strong>Step 5:</strong> Populating total_bounces in email_sales_funnel_report_daily");

    $sql = "
    UPDATE email_sales_funnel_report_daily d
    JOIN (
        SELECT 
            mbe.email_id, 
            DATE(mbe.created_at) AS calendar_date, 
            COALESCE(ml.latest_order_status, 'Unknown') AS latest_order_status, 
            COUNT(*) AS total_bounces
        FROM mautic_bounce_by_webhook_events mbe
        JOIN mautic_leads ml ON mbe.lead_id = ml.id
        WHERE mbe.bounce_id IS NOT NULL  -- Ensure only valid bounces
        GROUP BY mbe.email_id, DATE(mbe.created_at), ml.latest_order_status
    ) AS bounce_data
       ON d.email_id = bounce_data.email_id
      AND d.calendar_date = bounce_data.calendar_date
      AND d.latest_order_status = bounce_data.latest_order_status
    SET d.total_bounces = bounce_data.total_bounces;
    ";

    if ($conn->query($sql)) {
        logCustomMessage("✅ Successfully updated total_bounces.");
    } else {
        logCustomMessage("❌ SQL Error updating total_bounces: " . $conn->error);
    }
}

// -----------------------------------------------
// Step 6: Populate total_unique_clickers
// -----------------------------------------------
function populateUniqueClickers($conn) {
    logCustomMessage("<strong>Step 6:</strong> Populating total_unique_clickers in email_sales_funnel_report_daily");

    $sql = "
    UPDATE email_sales_funnel_report_daily d
    JOIN (
        SELECT 
            pcd.email_id, 
            DATE(pcd.click_date) AS calendar_date, 
            COALESCE(ml.latest_order_status, 'No Order') AS latest_order_status, 
            COUNT(DISTINCT pcd.lead_id) AS total_unique_clickers
        FROM parsed_click_data pcd
        LEFT JOIN mautic_leads ml ON pcd.lead_id = ml.id
        WHERE pcd.click_date IS NOT NULL
        GROUP BY pcd.email_id, DATE(pcd.click_date), ml.latest_order_status
    ) clickers_data
    ON d.email_id = clickers_data.email_id 
    AND d.calendar_date = clickers_data.calendar_date 
    AND d.latest_order_status = clickers_data.latest_order_status
    SET d.total_unique_clickers = clickers_data.total_unique_clickers;
    ";

    return $conn->query($sql) ? logCustomMessage("✅ Successfully updated total_unique_clickers.") 
                              : logCustomMessage("❌ SQL Error updating total_unique_clickers: " . $conn->error);
}


// -----------------------------------------------
// Step 7: Populate Remaining Indicators (To be implemented later)
// -----------------------------------------------
/*
function populateRemainingIndicators($conn) {
    logCustomMessage("<strong>Step 7:</strong> Populating remaining indicators (Not implemented yet)");
}
*/

// -----------------------------------------------
// Execute All Steps
// -----------------------------------------------
truncateDailyReport($conn);
insertDailyFunnelRows($conn);
populateBasicDailyFunnelValues($conn);
updateStatusStageType($conn);
populateBounces($conn);
populateUniqueClickers($conn);

// Step 7 will be implemented later
// 
// populateRemainingIndicators($conn);

// Close connection
$conn->close();
logCustomMessage("✅ <strong>Process completed successfully.</strong>");
?>
